; SPI_digipot_int_504_rev2020.a51		SPI 16-bit write, for digital pot, MCS4XXX IC
; portpins reconciled with text use, Sept 2020: all on P0,not P1
;  following Maxim App. Note 802
; reads and write using SPI bus: this Master drives CS and SCK;
; uses on-chip RAM locations to hold data out; one of these
; (gain control value) is brought in from keypad
; Modified 2/5/04 to do a single write on interrupt (to limit noise at PGA)
; Uses old PGA code, modified for different command code required by digital pot IC

    $NOSYMBOLS        ; keeps listing short

; default Ride7 path, Mar. 16
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\REG320.INC) 
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\vectors320.INC) 
	
	SCK EQU P0.2
	MISO EQU P0.5
	MOSI EQU P0.6
	CS EQU P0.7
	
	COMMAND EQU 11h	; this is command byte that tells digi pot that next byte is command
	
	SOFTFLAG EQU 0		; this is bit zero, flag set by ISR (this bit lives at address 20h!)
	STACKBOT EQU 07Fh	; puts stack at start of scratch RAM
	
; the following two internal-RAM locations are chosen to be 
;  bit-addressable, to speed up the processing

	MASTER_IN EQU 22h		; this is where inputs land
	MASTER_OUT EQU 23h	; ...and this is place whence outputs are drawn
	
;	R0 will count bits sent or received	

	ORG 0H
	LJMP SPI_IO

	ORG 0400h
				
SPI_IO: MOV SP, #STACKBOT
	ACALL INITS


HANG:	JNB SOFTFLAG,HANG			; linger here till interrupt tells us time to send gain value	
	CLR SOFTFLAG
	MOV MASTER_OUT, #11h			; set up instruction value to send, first of 2 bytes (this is a CONSTANT)
										;  this instruction means that following byte is gain value
										
	ACALL SEND_BYTE				; this FIRST invocation sends fixed code that means 
										;  "next byte will set GAIN"
										
	ACALL GET_POTVAL					; go pick up gain value from keypad	(this is the varying GAIN setting)													


	
	ACALL SEND_BYTE
	SETB CS							; disassert CS*, to signal end of 2-byte Xmssn
	SJMP HANG						; do this again, when interrupted again


;----SUBROUTINES------

SEND_BYTE:	SETB MISO	; Float the INPUT bit: this is a fake high: just makes sure the bit is not
							;  driven LOW: fake high is just disassertion, nearest
							;  the 8051 can come to floating its I/O pin
			CLR SCK		; start clock low (uses rising edge)
			CLR CS		; asserting this CS* (active low) starts the transfer
			
NEWPASS:	MOV R0, #8				; init counter for bits transferred				
			
; output from MASTER
XFER:	MOV C,MASTER_OUT.7
	MOV MOSI, C			; 2-step tfr from register to MASTER output port bit
	SETB SCK				; now clock that data out
	
;-----------------INPUT: omit for simplest first test------	
; input to MASTER
	MOV C,MISO		; now OK to sample input bit
	MOV MASTER_IN.7,C		; ...into input data location
	MOV A, MASTER_IN		; Now here's a 3-step rotate 
	RL	A
	MOV MASTER_IN, A		; ...and then restore after rotate.	
;----------------------		
	
	MOV A, MASTER_OUT		; Now a 3-step rotate
	RL A						; (8-bit rotate: not through carry)
	MOV MASTER_OUT, A

	CLR SCK					; now we've sent and picked up one bit. Drop clock.

	DJNZ R0,XFER			; till 8 are sent, keep sending bits

;	SETB CS					; disassert CS now that 8 bits are done
								; NO: we'll not dissassert till all 16-bits are sent
; these next 3 lines do nothing unless the peripheral talks back, as digipot does NOT	
	MOV A, MASTER_IN		; ...and display whatever came in
	MOVX @DPTR, A			; to display
	RET
	


INITS:  	MOV DPTR, #8000h	; point to displays
; ----NOW ENABLE INTERRUPTS----
	SETB IT0		; make INT0 Edge-sensitive	(p. 22)
	SETB EX0		; ...and enable INT0
	SETB EA		; Global int enable        (pp.31-32)
	RET
	
; -----GET GAIN---------
GET_POTVAL:	MOV DPTR, #8000h	; point to keypad
	MOVX A, @DPTR			; pick up key value
   CPL A						; flip every bit (because low value ==> hi gain)
	MOV MASTER_OUT,A 		; set up value to send
	RET	
	
;---ISR----
	ORG INT0VECTOR
	
	SETB SOFTFLAG			; tell Main program it's time to send a command
	RETI
		
		

							
END
	
